'' +--------------------------------------------------------------------------+
'' | SphinxOS   An Operating System for the Propeller Chip             v0.xxx |
'' +--------------------------------------------------------------------------+
'' |  Authors:                                                                |
'' |  (c) 2009,2010      Michael Park (mpark)             (original Sphinx)   |
'' |                     Radical Eye Software (rokicki?)  (fsrw objects)      |
'' |                     Lonesock & Rokicki               (fsrw objects)      |
'' |                     Andy Schenk (Ariba)              (sxkb_pc, sxtv_pc)  |
'' |                     Ray Rodrick (Cluso99)            (TriBlade/RamBlade) |
'' |                     Ray Rodrick (Cluso99)            (addn's to SphinxOS)|
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
''
''  Simple command line
''
''2009
''April
''  15    Adding file transfer to/from PC via serial
''  16    0.02
''        SendFile bugfix (while blksize == 1024)
''        Tries to run .bin before .eep now.
''June
''  4     Changing name from cl to sphinx. Converting to use sxfs, sxtv.
''  15    sxkb; adding c and cl commands
''  23    Corrected clock mode calculation.
''
''Juli 2009
'' changed for (Prop-)Terminal instead TV and PS2 by Andy Schenk
'' added GET <file> and PUT <file> for File up/download (not working yet)
''
'' RR20100117 Cluso99: allow specific pins for SD card (passed in basepin)
'' RR20100119 Cluso99: rename SphinxOS_xxx.spin; add isx_in.spin and isx.out.spin for standard I/O
'' RR20100203          v007 failed so retry taking 006 --> 006a
'' RR20100203          pass rendezvous at start
'' RR20100217          v006e
''                     v006n AuxIn/AuxOut 1-pin Kbd & TV working
'' RR20100222          v006q add OPTION to select the I/O drivers for StdIn/StdOut & AuxIn/AuxOut
'' RR20100222          v006s modify "sxfs2.spin" to only load 56 sectors to hub ram (was 63)
''                            this prevents overwriting the screen buffer & rendezvous area
''                     v006t add OPTIONs 2 & 3
''                     v006u RamBlade - kbd does not work at 6.5MHz, ok at 6MHz
'' RR20100224          v011  RamBlade (kbd not wkg @ 6.5MHz - to fix later), back to 6MHz



'WARNING: Somewhere there is a dependancy preventing relocation of upper hub definitions.

' 1. 1-pin TV tears display on fast scrolling
' 2. loading binary clears all hub ram. Needs to stop before clearing screen buffers & rendezvous.


CON
' Drivers...          Std Input       Std Output      Aux Input       Aux Output
' ------------        --------------  --------------  --------------  --------------
  OPTION = 2    ' 0 = PC-Serial       PC-Serial       1-pin Keyboard  1-pin TV
                ' 1 = 1-pin Keyboard  1-pin TV        PC-Serial       PC-Serial
                ' 2 = PC-Serial       PC-Serial       n/a             n/a
                ' 3 = 1-pin Keyboard  1-pin TV        n/a             n/a

  _clkfreq = hw#_clkfreq
  _clkmode = hw#_clkmode

  SDSPIRENDEZVOUS = hub#SDSPIRENDEZVOUS
  SXFS2RENDEZVOUS = hub#SXFS2RENDEZVOUS
  SXFSRENDEZVOUS  = hub#SXFSRENDEZVOUS
  METADATABUFFER  = hub#METADATABUFFER
       
OBJ
  hw    : "_hwdef"              'hardware pin definitions etc
  hub   : "_hubdef"             'fixed hub definitions ($7FFF... down)

  term  : "isx_out"             'StdOut standard output interface driver
  kb    : "isx_in"              'StdIn  standard input  interface driver
  auxo  : "isx_out"             'AuxOut standard output interface driver
  auxi  : "isx_in"              'AuxIN  standard input  interface driver


'  term: "sxtv_pc"
  sio   : "sx_pcsio"                ' PC-Serial I/O Driver  - cog I/O    driver
  optv  : "OnePinTVText_rr040k"     ' 1-pin TV Driver       - cog Output driver
  opkb  : "OnePinKeyboard_rr126"    ' 1-pin Keyboard Driver - cog Input  driver

  sxfs  : "sxfs"                    ' SD driver(s)          - cog(s) SD driver(s)
  f     : "sxfile"                  ' SD driver(s)          - cog(s) SD driver(s)

  fdx : "fullduplexserial"

DAT

ProgID  byte  "===== SphinxOS v011c =====",13,0


var
  long  cog
  long  buff1

PUB Main | err, p, ch, firsttime

{
  fdx.start(31,30,0,115200)

  waitcnt(clkfreq*5 + cnt)
  fdx.str(string(13,"Test 1-pin Keyboard v120/006au",13))
  fdx.str(string("Press <space> to initialise"))

'  buff1 := hw#ktime << 16 | hw#kdpin                           'kbd data pin & bit time (estimate) 0=wait for a key
'  cog := cognew(@entry, @buff1) + 1

  long[hub#_auxIn]  := hw#ktime << 16 | hw#kdPin          ' pass kbd time and pin#
  opkb.start(hub#_auxin)                                  ' StdIN  cog driver (1-pin Keyboard Driver)


  repeat until hub#_auxIn == 0                                'wait until cleared

  repeat
    if long[hub#_auxIn] <> 0                                        'wait for a char
      ch := long[hub#_auxIn]                                         'get char
      long[hub#_auxIn] := 0                                         ' and clear
      if ch < $80                                        'strip function keys, etc
        fdx.tx(ch)                                       'output char
}

  waitcnt(clkfreq*2 + cnt)
  
  'start standard output driver call...
  term.start(hub#_StdOut)                               ' StdOut driver
  'start standard input  driver call...
  kb.start(hub#_StdIn)                                  ' StdIn  driver

  if OPTION == 0 or OPTION == 1                         ' AUXILLARY PORT EXISTS
    'start auxillary output driver call...
    auxo.start(hub#_AuxOut)                             ' AuxOut driver
    'start auxillary input  driver call...
    auxi.start(hub#_AuxIn)                              ' AuxIn  driver

'--------------------------------------------------------------------------------------------------
  if OPTION == 0 or OPTION == 2                         ' OPTION = 0 & 2 FOR PC AS STANDARD I/O
    'start standard input/output driver...
    firsttime := sio.start(hub#_StdIn,hub#_StdOut,hw#serPins,hw#baud)   ' StdIn & StdOut cog driver (PC-Serial I/O Driver)
  elseif OPTION == 1                                    ' OPTION = 1 FOR PC AS AUXILLARY I/O
    'start AUX input/output driver...
    firsttime := sio.start(hub#_AuxIn,hub#_AuxOut,hw#serPins,hw#baud)   ' AuxIn & AuxOut cog driver (PC-Serial I/O Driver)
  else
    'check if Standard Output driver running
    long[hub#_StdOut] := $47 | %100                     ' send BELL  "G"
    waitcnt(clkfreq/10 + cnt)                           ' wait 100ms
    if long[hub#_StdOut]                                ' char not taken?
      firsttime := true
      long[hub#_StdOut] := 0
    else                                                ' taken, so cog running
      firsttime := false
'--------------------------------------------------------------------------------------------------
  if firsttime                                          ' START TV & KBD COG DRIVERS (if required)...
    if OPTION == 1 or OPTION == 3                       ' OPTION = 1 & 3 FOR 1-PIN KBD & TV AS STANDARD I/O
      'start standard output driver...
      long[hub#_StdOut] := hub#_Screen << 8 | hw#tvPinc       ' pass hub screen address & tv pin#
      optv.start(hub#_StdOut)                                 ' StdOut cog driver (1-pin TV Driver)
      'start standard input  driver...
      long[hub#_StdIn]  := hw#ktime << 16 | hw#kdPin          ' pass kbd time and pin#
      opkb.start(hub#_StdIn)                                  ' StdIN  cog driver (1-pin Keyboard Driver)
    elseif OPTION == 0                                  ' OPTION = 0 FOR 1-PIN KBD & TV AS AUXILLARY I/O
      'start aux output driver...
      long[hub#_AuxOut] := hub#_Screen << 8 | hw#tvPinc       ' pass hub screen address & tv pin#
      optv.start(hub#_AuxOut)                                 ' AuxOut cog driver (1-pin TV Driver)
      'start aux input  driver...
      long[hub#_AuxIn]  := hw#ktime << 16 | hw#kdPin          ' pass kbd time and pin#
      opkb.start(hub#_AuxIn)                                  ' AuxIN  cog driver (1-pin Keyboard Driver)
'--------------------------------------------------------------------------------------------------
    'firsttime

    term.out(0)
    term.out(13)                                        'because 0 doesnt clear screen in some drivers
    term.str(@ProgID)
    term.str(string("SD="))
    term.dec(hw#spiDO)
    term.out(",")
    term.dec(hw#spiCLK)
    term.out(",")
    term.dec(hw#spiDI)
    term.out(",")
    term.dec(hw#spiCS)

    if OPTION == 0 or OPTION == 2                        ' OPTION = 0 & 2 FOR PC AS STANDARD I/O
      term.str(string(" Serial="))
      term.dec(hw#rxPin)
      term.out(",")
      term.dec(hw#txPin)
    else
      term.str(string(" Kbd/TV="))
      term.dec(hw#kdPin)
      term.out(",")
      term.dec(hw#TVPinc)
    if OPTION == 1                                      ' AUX = Serial
      term.str(string(" Serial2="))
      term.dec(hw#rxPin)
      term.out(",")
      term.dec(hw#txPin)
    if OPTION == 0                                      ' AUX = KBD/TV
      term.str(string(" Kbd/TV2="))
      term.dec(hw#kdPin)
      term.out(",")
      term.dec(hw#TVPinc)
    term.out(13)


    if OPTION == 1 or OPTION == 3
      term.out(0)                                             'clear screen
      term.str(string("1-pin TV & Keyboard SphinxOS v0.006n",13,"...by Cluso99 - enjoy :-)",13,13))
      term.str(string("Hit <spacebar> to synchronise keyboard",13))
      term.str(string(" keyboard will respond with <cr>"))

      'the <cr> and bit time displays are optional parameters set in CON of the OnePinKeyboard driver.
      ch := kb.in                                             'get the <cr>
      term.out(ch)
      ch := kb.in                                             'get the calculated bit time / 64
      ch := ch << 6
      term.str(string("Keyboard bit time = "))
      term.dec(ch)

    if OPTION == 0
      auxo.out(0)                                             'clear screen
      auxo.str(string("1-pin TV & Keyboard SphinxOS v0.006n",13,"...by Cluso99 - enjoy :-)",13,13))
      auxo.str(string("Hit <spacebar> to synchronise keyboard",13))
      auxo.str(string(" keyboard will respond with <cr>"))

      'the <cr> and bit time displays are optional parameters set in CON of the OnePinKeyboard driver.
      ch := auxi.in                                           'get the <cr>
      auxo.out(ch)
      ch := auxi.in                                           'get the calculated bit time / 64
      ch := ch << 6
      auxo.str(string("Keyboard bit time = "))
      auxo.dec(ch)


    if OPTION == 0 or OPTION == 1                       ' AUXILLARY PORT EXISTS
      auxo.out(0)
      auxo.str(@ProgID)
'--------------------------------------------------------
    'firsttime
    'start SD driver(s)
    sxfs.start(hw#sdPins)                               ' SD cog Driver(s)
'--------------------------------------------------------
  if not firsttime
    'msg to show we are here
    term.out(13)
    term.str(@ProgID)
    if OPTION == 0 or OPTION == 1                       ' AUXILLARY PORT EXISTS
      term.out(13)
      auxo.str(@ProgID)
'--------------------------------------------------------

  repeat
    f.Close
    err := \DoIt
    if err
      if err > 0
        term.str( err )
      else
        term.str( string("Error ") )
        term.dec( err )
      term.out( 13 )
    else
      term.str( string("Normal exit") )                 ' This never happens


pri DoIt | ch
  term.str(string( "SD>_",8 ))
  repeat
    if kb.peek
      EditCmdLine
      Execute
      term.str(string( "??>_",8 ))                 '<==== never executed

con
  MAXCMDLINELENGTH = 78 ' some arbitray limit.
  CR     = $0d
  BKSP   = $c8

var
  byte cmdLine[MAXCMDLINELENGTH+1]                      ' +1 for null terminator
  long cmdLineLength

pri EditCmdLine | key
  cmdLineLength~                ' cursor is always at end of command line

  repeat
    case key := kb.in
      $20..$7f:
        if cmdLineLength < MAXCMDLINELENGTH
          cmdLine[ cmdLineLength++ ] := key
          term.out( key )
          term.out( "_" )        
          term.out( 8 )        
      CR:
        term.out( " " )
        term.out( 13 )
        cmdLine[ cmdLineLength ]~
        return true
      BKSP:
        if cmdLineLength
          --cmdLineLength
          term.out( " " )
          term.out( 8 )
          term.out( 8 )
          term.out( "_" )
          term.out( 8 )

con
TIMEOUT = 500

var
  byte buffer[1024]
       
pri Execute | n, p, q, run, c, b, len, i
  c~
  ifnot n := ParseCmdLine
    return
  p := @cmdLine
  ToUpper(p)
  
  if run := strcomp( @cmdLine, string("RUN") ) 
    --n
    p += strsize(p) + 1
  elseif strcomp( @cmdLine, string("C") )
    c := 1
    --n                                                                                                               
    p += strsize(p) + 1                                                                     ' command arg1 arg2...    
  elseif strcomp( @cmdLine, string("CL") )                                                  ' run command arg1 arg2...
    c := 2                                                                                  ' c arg1 arg2...          
    --n                                                                                     ' cl arg1 arg2...         
    p += strsize(p) + 1

  'added by AS
  elseif strcomp( @cmdLine, string("EXIT") )
    reboot
  elseif strcomp( @cmdLine, string("GET") )
    p += strsize(p) + 1
    n := strsize(p)
    if n
      q := p
      if byte[q] == "*"
        q += strsize(q) + 1
'      if f.open( q,"W") <> 0  or strsize(p)==0
'        abort string(" - can't open")
      term.out(0)
      term.str(p)                           'send filename
      term.str(string(5,$82))               'initiate download
      term.out(n)
      term.out("f")
      repeat 1000
        if b := kb.peek
          quit
        waitcnt(clkfreq>>9+cnt)
      ifnot b
        abort string(" - File not found")
      len := kb.in                         'PropTerm sends first the FileLength (4 bytes)
      b := kb.in
      len |= b<<8
      b := kb.in
      len |= b<<16
      b := kb.in
      len |= b<<24
      n :=  0                              'counter for received bytes
      repeat while n < len
        term.out(".")                      'trigger next 16 bytes
        repeat i from 0 to 15              'receive 16 bytes
          if n+i => len                    'quit if length reached
            quit
          b := kb.in
          buffer[i] := b                   'store in buffer
'        f.write(@buffer,16)                'write the buffer to the SD card
        n += 16
'      f.close
      term.out(13)
      term.dec( n )
      abort string(" bytes received", 13)
  elseif strcomp( @cmdLine, string("PUT") )
    p += strsize(p) + 1
    abort string(" not yet implemented", 13)
    
  ifnot n                                                               
    abort string("No file specified")                                   
                                                                        
  f.Open( string("args.d8a"), "W" )                     ' write args    
  if c == 0
    q := p + strsize(p) + 1                             ' for run and plain command, skip first arg (command name)
    --n
    f.Write( @n, 1 )
    repeat n
      f.Write( q, strsize(q) + 1 )
      q += strsize(q) + 1
  else                                                  ' for c and cl,
    ++n                                                 '  add extra arg (/c or /l)
    f.Write( @n, 1 )
    repeat n-1
      f.Write( p, strsize(p) + 1 )
      p += strsize(p) + 1
    if c == 1
      f.Write( string("/c"), 3 )
    else
      f.Write( string("/l"), 3 )
        
  n~
  f.Write( @n, 1 )              ' final terminating null    
  f.Close

  if c 
    if f.Open( string("lex.bin"), "R" ) <> 0
      abort string("lex.bin not found")
    f.Execute( 0 )  
    

  if EndsWith( p, string(".BIN") ) or EndsWith( p, string(".EEP") )
    if f.Open( p, "R" ) <> 0
      term.str( p )
      abort string(" -- not found")
  else
    if strsize(p) > MAXFILENAMELENGTH - 4
      term.str( p )
      abort string(" -- filename too long")
    bytemove( @filename, p, strsize(p) + 1 )
    bytemove( @filename + strsize(p), string(".BIN"), 5 )
    if f.Open( @filename, "R" ) <> 0
      bytemove( @filename + strsize(p), string(".EEP"), 5 )
      if f.Open( @filename, "R" ) <> 0
        term.str( p )
        abort string(" -- no .bin or .eep found")
  f.Execute( run )

pri ToUpper( p )
  repeat while byte[p]
    if "a" =< byte[p] and byte[p] =< "z"
      byte[p] += "A" - "a"
    ++p

pri EndsWith( p, s )
  if strsize(p) < strsize(s)
    return false
  p += strsize(p) - strsize(s)
  repeat while byte[p]
    if byte[p++] <> byte[s++]
      return false
  return true

con
  MAXFILENAMELENGTH = 12 ' 8 + . + 3
  FILENAMEBUFFERSIZE = MAXFILENAMELENGTH + 1 ' 8 + . + 3 + null
  CMDNAMEBUFFERSIZE = 9 ' 8 + null
  
var
  byte filename[MAXFILENAMELENGTH+1] ' 8 + . + 3 + null
  
pri ParseCmdLine : n | pSrc, pDst, state
{{
  Converts cmdLine from " ab  de   fg  \" to "ab\de\fg\" ('\' = null).
  Returns # of "words".
}}
{
  state     whitespace                null                      other char
  0         ++pSrc;                   append \0; quit           append char; => 1
  1         append \0; ++n; => 0      append \0; ++n; quit      append char;
}
  pSrc := @cmdLine
  pDst := @cmdLine
  state~
  repeat
    if state == 0
      if byte[pSrc] == " "
        ++pSrc
      elseif byte[pSrc] == 0
        byte[pDst++]~
        quit
      else
        byte[pDst++] := byte[pSrc++]
        state := 1
    elseif state == 1
      if byte[pSrc] == " "
        ++pSrc
        byte[pDst++]~
        ++n
        state~
      elseif byte[pSrc] == 0
        byte[pDst++]~
        ++n
        quit
      else
        byte[pDst++] := byte[pSrc++]

{{
Copyright (c) 2009 Michael Park
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    |
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
        
